package sbc.mgclus.parser;

import java.io.*;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;
import org.jgrapht.*;
import org.jgrapht.graph.*;

import sbc.mgclus.algorithm.ClusteringKL;
import sbc.mgclus.data.*;

public class GraphTSVParser {
	private HashMap<String, Cluster> vertexIndex;
	private Logger logger=Logger.getLogger(ClusteringKL.class);

	public WeightedGraph<Cluster, DefaultWeightedEdge> parseGraph(File graphFile, Boolean useWeights, HashSet<String> seedList) {
		WeightedGraph<Cluster, DefaultWeightedEdge> graph = new SimpleWeightedGraph<Cluster,DefaultWeightedEdge>(DefaultWeightedEdge.class);
		try {
			FileReader fileReader = new FileReader(graphFile);
			BufferedReader reader = new BufferedReader(fileReader);
			logger.info(String.format("read graph file: %s...\n", graphFile.getName()));
			vertexIndex = new HashMap<String, Cluster>();
			Cluster cluster1;
			Cluster cluster2;
			
			String line = null;
			
			int clusterID = 1;
			while ((line = reader.readLine()) != null){
				String[] lineList = line.split("\\s");
				if (lineList.length >= 2){
					if (! lineList[0].equals(lineList[1])){
						if (!vertexIndex.containsKey(lineList[0])){
							Vertex v1 = new Vertex (lineList[0], clusterID);
							cluster1 = new Cluster(clusterID);
							cluster1.addMember(v1);
							graph.addVertex(cluster1);
							vertexIndex.put(lineList[0], cluster1);
							clusterID += 1;
							if (seedList.contains(lineList[0]))
								v1.setSeed(true);
								cluster1.setSeedCount(1);
						}
						else
							cluster1 = vertexIndex.get(lineList[0]);
						if (!vertexIndex.containsKey(lineList[1])){
							Vertex v2 = new Vertex (lineList[1], clusterID);
							cluster2 = new Cluster(clusterID);
							cluster2.addMember(v2);
							graph.addVertex(cluster2);
							vertexIndex.put(lineList[1], cluster2);
							clusterID += 1;
							if (seedList.contains(lineList[1]))
								v2.setSeed(true);
								cluster2.setSeedCount(1);
						}
						else
							cluster2 = vertexIndex.get(lineList[1]);
						graph.addEdge(cluster1, cluster2);
						if (lineList.length >= 3 && useWeights){
							Double edgeWeight;
							try{
								edgeWeight = Double.valueOf(lineList[2].trim()).doubleValue();
							}catch (NumberFormatException e) {
								edgeWeight = 1.0;
							}
							graph.setEdgeWeight(graph.getEdge(cluster1, cluster2), edgeWeight);
						}
						else
							graph.setEdgeWeight(graph.getEdge(cluster1, cluster2), 1.0);
					}
				}
			}
			//logger.info(String.format("finished reading the graph file...\n"));
			reader.close();
			
		}catch (FileNotFoundException ex){
			logger.error("Graph file does not exists\n");
			//ex.printStackTrace();
			System.exit(1);
		}catch (Exception ex) {
			logger.error("an error occurred parsing the graph file\n");
			//ex.printStackTrace();
			System.exit(1);
		}
		return graph;
	}
	
}
